// BrdfDlg.cpp : implementation file
//

#include "stdafx.h"
#include "IME.h"
#include "BrdfDlg.h"
#include ".\brdfdlg.h"
#include "ProjMgr.h"
#include "ImgMatEdit.h"

// CBrdfDlg dialog

IMPLEMENT_DYNAMIC(CBrdfDlg, CDialog)
CBrdfDlg::CBrdfDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CBrdfDlg::IDD, pParent)
{
}

CBrdfDlg::~CBrdfDlg()
{
}

void CBrdfDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_VIEW_DIS_SLIDER, m_view_dis_slider);
	DDX_Control(pDX, IDC_RHO_S_SLIDER, m_rho_s_slider);
	DDX_Control(pDX, IDC_RHO_D_SLIDER, m_rho_d_slider);
	DDX_Control(pDX, IDC_RMS_SLIDER, m_rms_slider);
}


BEGIN_MESSAGE_MAP(CBrdfDlg, CDialog)
	ON_WM_PAINT()
	ON_BN_CLICKED(IDC_BRDF_APPLY, OnBnClickedBrdfApply)
	ON_BN_CLICKED(IDC_BRDF_SET_COLOR, OnBnClickedBrdfSetColor)
END_MESSAGE_MAP()

// CBrdfDlg message handlers

BOOL CBrdfDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	rho_d		= (float)0.290;
	rho_s		= (float)0.083;
	rms			= (float)0.082;
	view_dis	= (float)600;

	m_view_dis_slider.SetRange(0, 1000);
	m_rho_d_slider.SetRange(0, 1000);
	m_rho_s_slider.SetRange(0, 1000);
	m_rms_slider.SetRange(1, 1000);

	m_view_dis_slider.SetPos(int(view_dis-300));
	m_rho_d_slider.SetPos(int(rho_d * 1000));
	m_rho_s_slider.SetPos(int(rho_s * 1000));
	m_rms_slider.SetPos(int(rms * 5000));

	r = g = b = 1.0;
	return true;
}

void CBrdfDlg::OnPaint()
{
	CPaintDC dc(this); // device context for painting
	CStatic *control = NULL;
	CDC* pdc = NULL;
	CRect r;

	RgbImage & outimg = g_ImgMatEdit.m_brdf_previmg;

	if( outimg.m_width != 0 && outimg.m_height != 0){
		control = (CStatic*)GetDlgItem(IDC_BRDFIMG);
		control->GetClientRect(&r);

		float rate1 = (r.right-2) /(float)outimg.m_width;
		float rate2 = (r.bottom-2)/(float)outimg.m_height;
		float rate = min(rate1, rate2);
		if(rate > 1.0f)
			rate = 1.0f;

		int DestW = (int)(rate * outimg.m_width);
		int DestH = (int)(rate * outimg.m_height);
		int DestX = (r.right-2)/2 - DestW/2;
		int DestY = (r.bottom-2)/2 - DestH/2;

		pdc = control->GetDC();
		::SetStretchBltMode(pdc->m_hDC, COLORONCOLOR);
		::StretchDIBits(pdc->m_hDC,
			DestX,
			DestY + DestH,
			DestW,
			-DestH,
			0,
			0,
			outimg.m_width,
			outimg.m_height,
			(void*)m_abmp.GetAlignedData(outimg),
			&getBitmapInfo(outimg),
			DIB_RGB_COLORS,
			SRCCOPY);
		ReleaseDC(pdc);
	}
}

void CBrdfDlg::ReBRDF()
{
	bool result;

	view_dis	= float(m_view_dis_slider.GetPos() + 300);
	rho_d		= float(m_rho_d_slider.GetPos() / 1000.0);
	rho_s		= float(m_rho_s_slider.GetPos() / 1000.0);
	rms			= float(m_rms_slider.GetPos() / 5000.0);

	g_ImgMatEdit.brdf.set_color(r, g, b);
	g_ImgMatEdit.brdf.set_parameters(rho_d, rho_s, rms);
	result = g_ImgMatEdit.brdf.apply_brdf(view_dis);
	if(result == true)
	{
		g_ImgMatEdit.brdf.make_preview();
	}
	RedrawWindow();
}

void CBrdfDlg::OnBnClickedBrdfApply()
{
	ReBRDF();
}

void CBrdfDlg::OnBnClickedBrdfSetColor()
{
	CColorDialog cdlg(RGB( (BYTE)(r*255), (BYTE)(g*255), (BYTE)(b*255)), CC_FULLOPEN);
	if( cdlg.DoModal() == IDOK)
	{
		COLORREF c = cdlg.GetColor();
		r = GetRValue(c)/255.0f;
		g = GetGValue(c)/255.0f;
		b = GetBValue(c)/255.0f;
	}

	ReBRDF();
}
