#include "dip_convert.h"
#include "math_funcs.h"

namespace dip{
	namespace convert{
		FloatRgbImage ToFloatImage(RgbImage & rh){
			FloatRgbImage result(rh.m_width, rh.m_height);
			int size = rh.m_width * rh.m_height * 3;
			for(int i = 0; i < size; i++){
				((float*)result.m_buf)[i] = ((unsigned char*)rh.m_buf)[i]/255.0f;
			}
			return result;
		}

		FloatImage    ToFloatImage(GrayImage & rh){
			FloatImage result(rh.m_width, rh.m_height);
			int size = rh.m_width * rh.m_height;
			for(int i = 0; i < size; i++){
				result.m_buf[i] = rh.m_buf[i]/255.0f;
			}
			return result;
		}

		GrayImage     ToNormalImage(FloatImage & rh){
			GrayImage result(rh.m_width, rh.m_height);
			int size = rh.m_width * rh.m_height;
			for(int i = 0; i < size; i++){
				if( rh.m_buf[i] > 1.0f)
					result.m_buf[i] = 255;
				else if( rh.m_buf[i] < 0.0f)
					result.m_buf[i] = 0;
				else
					result.m_buf[i] = math::int_round(rh.m_buf[i]*255);
			}
			return result;
		}

		RgbImage      ToNormalImage(FloatRgbImage & rh){
			RgbImage result(rh.m_width, rh.m_height);
			int size = rh.m_width * rh.m_height * 3;
			for(int i = 0; i < size; i++){
				float &y = ((float*)rh.m_buf)[i];
				unsigned char &x = ((unsigned char*)result.m_buf)[i];
				if( y > 1.0f)
					x = 255;
				else if( y < 0.0f)
					x = 0;
				else
					x = math::int_round(y*255);
			}
			return result;
		}

		GrayImage ToIntensity(RgbImage & rh){
			GrayImage result(rh.m_width, rh.m_height);

			int size = rh.m_width*rh.m_height;
			for(int i = 0; i < size; i++){
				int X = math::int_round(0.299f*rh.m_buf[i].R + 0.587f*rh.m_buf[i].G + 0.114f*rh.m_buf[i].B);
				if( X > 255)
					result.m_buf[i] = 255;
				else if( X < 0)
					result.m_buf[i] = 0;
				else
					result.m_buf[i] = (unsigned char)X;
			}
			return result;
		}


		FloatImage ToIntensity(FloatRgbImage & rh){
			FloatImage result(rh.m_width, rh.m_height);
			int size = rh.m_width*rh.m_height;
			for(int i = 0; i < size; i++){
				result.m_buf[i] = 0.299f*rh.m_buf[i].R + 0.587f*rh.m_buf[i].G + 0.114f*rh.m_buf[i].B;
			}
			return result;
		}

		FloatRgbImage ToRgb(FloatImage & rh){
			FloatRgbImage result(rh.m_width, rh.m_height);
			for(int i = 0; i < rh.m_width; i++){
				for(int j = 0; j < rh.m_height; j++){
					result.entry(i,j).R = result.entry(i,j).G = result.entry(i,j).B = rh.entry(i,j);
				}
			}
			return result;
		}

		RgbImage      ToRgb(GrayImage & rh){
			RgbImage result(rh.m_width, rh.m_height);
			for(int i = 0; i < rh.m_width; i++){
				for(int j = 0; j < rh.m_height; j++){
					result.entry(i,j).R = result.entry(i,j).G = result.entry(i,j).B = rh.entry(i,j);
				}
			}
			return result;
		}


	}
}