#include "dip_image.h"
#include "dip_filter.h"
#include "math_funcs.h"
#include<vector>
using namespace std;
using namespace math;

namespace dip{
	namespace filter{
		const float window_var_ratio = 2.6f;

		vector<float> GaussianFilter1D(float variance){
			if( variance < 1e-9f){
				vector<float> u;
				u.push_back(1.0f);
				return u;
			}
				
			int n = int_round(window_var_ratio*variance);
			n = max(1, n);
			vector<float> weights1d;
			weights1d.resize(2*n+1);
			
			weights1d[n] = 1.0;

			for(int i = 1; i <= n; i++){
				weights1d[n+i] = weights1d[n-i] = (float)exp(-0.5*pow((double)i/variance, 2));
			}
			return weights1d;
		}

		vector<float> GaussianFilter1D(float variance, int side_width){
			int n = side_width;
			n = max(1, n);
			vector<float> weights1d;
			weights1d.resize(2*n+1);
			
			weights1d[n] = 1.0;

			for(int i = 1; i <= n; i++){
				weights1d[n+i] = weights1d[n-i] = (float)exp(-0.5*pow((double)i/variance, 2));
			}
			return weights1d;

		}

		vector<float> GaussianHalfSide(float variance, int side_width){
			int size = side_width + 1;
			vector<float> weights1d(size);

			for(int i = 0; i < size; i++){
				weights1d[i] = (float)exp(-0.5*pow((double)i/variance, 2));
			}

			return weights1d;
		}


		FloatImage GaussianFilter2D(float variance){
			int i, j;
			vector<float> gaussian1d = GaussianFilter1D(variance);
			FloatImage gaussian2d((int)gaussian1d.size(), (int)gaussian1d.size());
			double total = 0.0;
			for(i = 0; i < (int)gaussian1d.size(); i++){
				for(j = 0; j < (int)gaussian1d.size(); j++){
					gaussian2d.entry(i,j) = gaussian1d[i]*gaussian1d[j];
					total += gaussian2d.entry(i,j);
				}
			}

			for(i = 0; i < (int)gaussian1d.size(); i++){
				for(j = 0; j < (int)gaussian1d.size(); j++){
					gaussian2d.entry(i,j) /= (float)total;
				}
			}
			return gaussian2d;
		}
	}
}
