#include "dip_io.h"
#include "dip_routine.h"
using namespace dip::io;
using namespace dip::routine;

#include <fstream>
using namespace std;

#include <windows.h>


void dip::io::DumpGrayScaleToBmpfile(GrayImage & image, const char filename[]){
	WriteBmpFile(ConvertToRgb(image), filename);
}


RgbImage dip::io::ReadBmpFile(char * filename){
	RgbImage bitmap;
	ifstream file;
	BITMAPFILEHEADER FileHeader;
	BITMAPINFOHEADER InfoHeader;

	//open file 
	file.open(filename, ios::binary);
	if(!file)
		throw exception("the bitmap file doesn't exist!");


	/* ŪYT */
	file.read((char*)&FileHeader, sizeof(FileHeader));
	file.read((char*)&InfoHeader, sizeof(InfoHeader));

	/* Tw榡 */
	if( FileHeader.bfType != 'MB' )
		throw exception("The file is not bitmap file");
	if( InfoHeader.biCompression != 0 )
		throw exception("The file is compressed");
	if( InfoHeader.biBitCount != 24 )
		throw exception("The bitmap file is not 24bits format");

	/* oϼeιϰ */
	bitmap.resize(InfoHeader.biWidth, InfoHeader.biHeight);

	/* pCCݲL bytes  */
	int fix = 0;
	if( (bitmap.m_width*3)%4 != 0 )
		fix = 4 - ((bitmap.m_width*3)%4);

	/* ϤJUT */
	for( int i=bitmap.m_height-1; i>=0; i-- )
	{
		file.read((char*)&bitmap.entry(0,i), bitmap.m_width*3);

		//for( j=0; j<bitmap.m_width; j++ )
		//{
		//	fread(&rgb, sizeof(RGBTRIPLE), 1, fp);
		//	color[j][i].rgbtBlue=rgb.rgbtBlue;
		//	color[j][i].rgbtGreen=rgb.rgbtGreen;
		//	color[j][i].rgbtRed=rgb.rgbtRed;
		//}
		/* LUChlT */
		file.seekg(fix, ios::cur);
		//for(int n=0; n<fix; n++ )
		//	file.read(ByteBuf, sizeof(BYTE), 1, fp);
	}
	file.close();
	return bitmap;
}

void dip::io::WriteBmpFile(RgbImage & image, const char filename[]){
	BITMAPFILEHEADER bmfh;
	((char*)&bmfh.bfType)[0] = 'B';
	((char*)&bmfh.bfType)[1] = 'M';

	int fix = 0;
	if( (image.m_width*3)%4 != 0 )
		fix = 4 - ((image.m_width*3)%4);

	
	bmfh.bfSize = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + (image.m_width + fix)*image.m_height*sizeof(uRGB);
	bmfh.bfReserved1 = 0;
	bmfh.bfReserved2 = 0;
	bmfh.bfOffBits = sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER);

	BITMAPINFO info;
	ZeroMemory(&info, sizeof(info));
	info.bmiHeader.biBitCount = 24;
	info.bmiHeader.biHeight = image.m_height;
	info.bmiHeader.biWidth = image.m_width;
	info.bmiHeader.biPlanes = 1;
	info.bmiHeader.biCompression = BI_RGB;
	info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	info.bmiHeader.biSizeImage = 0;


	//else
	//	fix = 0;
	

	ofstream file;
	file.open(filename, ios::binary|ios::out|ios::trunc);
	if(file){
		file.write((char*)&bmfh, sizeof(BITMAPFILEHEADER));
		file.write((char*)&info.bmiHeader, sizeof(BITMAPINFOHEADER));
		
		for(int i = image.m_height - 1; i >= 0; i--){
			file.write((char*)&image.entry(0,i), image.m_width*sizeof(uRGB));
			for(int i = 0; i < fix; i++)
				file.put(0);
		}
		//file.write((char*)image.m_buf, image.m_width*image.m_height*sizeof(RGBpel));


		file.close();
	}
}

char* dip::io::GetAlignedRgbData(RgbImage & in,  char* buf){
	int w = in.m_width*3;
	int fix = 0;

	char* ptr = buf;

	if( w % 4 != 0){
		fix = 4 - (w%4);
	}

	for(int y = 0; y < in.m_height; y++){
		memmove((void*)ptr, (void*)&in.entry(0,y), w);
		ptr += w + fix;
	}

	return buf;
}

int dip::io::GetAlignedRgbDataSize(int w, int h){
	int buf_width = 0;

	if( w%4 != 0){
		buf_width = (4 - ((w*3)%4)) + w*3;
	}else{
		buf_width = w*3;
	}

	return buf_width*h;
}