#pragma once
#include "dip_image.h"
#include "math_vector.h"

namespace dip{

	template<class T>
	class Painter{
	public:
		Painter(Image<T> & I);
		Painter();

		void Bind(Image<T> & I);
		void Unbind();
		inline void SetColor( const T & icolor );
		inline bool IsBind();

	public:
		void DrawLine(int x1, int y1, int x2, int y2);
		void MarkPoint(int x, int y, int size = 2);

	private:
		Image<T> * m_ref;
		T color;

	};

	//-----------------------------------------------------------------------------------------------------------//
	// typedef
	//-----------------------------------------------------------------------------------------------------------//
	typedef Painter<float>         FloatPainter;
	typedef Painter<fRGB>          FloatRgbPainter;
	typedef Painter<unsigned char> GrayPainter;
	typedef Painter<uRGB>          RgbPainter;

	//-----------------------------------------------------------------------------------------------------------//
	// implementation
	//-----------------------------------------------------------------------------------------------------------//


	template<class T>
	Painter<T>::Painter<T>(Image<T> & I) : m_ref(&I){
	}

	template<class T>
	Painter<T>::Painter<T>() : m_ref(NULL){
	}

	template<class T>
	void Painter<T>::Bind(Image<T> & I){
		m_ref = &I;
	}

	template<class T>
	void Painter<T>::Unbind(){
		m_ref = NULL;
	}

	template<class T>
	bool Painter<T>::IsBind(){
		return (m_buf == NULL) ? false : true;
	}

	template<class T>
	void Painter<T>::SetColor(const T & icolor){
		color = icolor;
	}

	template<class T>
	void Painter<T>::MarkPoint(int x, int y, int size = 2){
		
		Image<T> & image = *m_ref;

		int cx, cy;

		cy = y - 2;
		for(cx = x - size; cx <= x + size ; cx ++){
			if( image.checkInside(cx, cy) ){
				image.entry(cx, cy) = color;
			}
		}

		cy = y + 2;
		for(cx = x - size; cx <= x + size ; cx ++){
			if( image.checkInside(cx, cy) ){
				image.entry(cx, cy) = color;
			}
		}


		cx = x - 2;
		for(cy = y - size; cy <= y + size; cy++){
			if( image.checkInside(cx , cy) ){
				image.entry( cx, cy ) = color;
			}
		}

		cx = x + 2;
		for(cy = y - size; cy <= y + size; cy++){
			if( image.checkInside(cx , cy) ){
				image.entry( cx, cy ) = color;
			}
		}
	}



	template<class T>
	void Painter<T>::DrawLine(int x1, int y1, int x2, int y2){
		Image<T> & image = *m_ref;


		int dx = x2 - x1;
		int dy = y2 - y1;
		int D;
		int x;
		int y;

		if( dx*dy >= 0){ // dx dy have the same sign
			if( dx < 0 ){  // dx and dy < 0 // swap point
				int tmp;
				tmp = x1;
				x1 = x2;
				x2 = tmp;
					
				tmp = y1;
				y1 = y2;
				y2 = tmp;

				dx = -dx;
				dy = -dy;
			}

			if( dx >= dy ){

				D = -dx;
				x = x1;
				y = y1;


				for(int i = x1; i <= x2; i++){
					if( !(x < 0 || x >= image.m_width - 1 || y < 0 || y >= image.m_height - 1) )
						image.entry(x,y) = color;
					x++;
					D += 2*dy;
					if( D > 0 ){
						y++;
						D -= 2*dx;
					}
				}
			}
			else{
				D = -dy;
				x = x1;
				y = y1;

				for(int i = y1; i <= y2; i++){
					if( !(x < 0 || x >= image.m_width - 1 || y < 0 || y >= image.m_height - 1) )
						image.entry(x,y) = color;
					y++;
					D += 2*dx;
					if( D > 0 ){
						x++;
						D -= 2*dy;
					}
				}
			}
		}
		else{
			if( (dx < 0 && abs(dx) >= abs(dy)) || (dx > 0 && abs(dx) < abs(dy)) ){  // dx < 0 and dy > 0 // swap point 
				int tmp;   
				tmp = x1;
				x1 = x2;
				x2 = tmp;
					
				tmp = y1;
				y1 = y2;
				y2 = tmp;

				dx = -dx;
				dy = -dy;
			}              //dy < 0 dx > 0

			if( abs(dx) >= abs(dy) ){

				D = -dx;
				x = x1;
				y = y1;


				for(int i = x1; i <= x2; i++){
					if( !(x < 0 || x >= image.m_width - 1 || y < 0 || y >= image.m_height - 1) )
						image.entry(x,y) = color;
					x++;
					D += 2*(-dy);
					if( D > 0 ){
						y--;
						D -= 2*dx;
					}
				}
			}
			else{
				D = -dy;
				x = x1;
				y = y1;

				for(int i = y1; i <= y2; i++){
					if( !(x < 0 || x >= image.m_width - 1 || y < 0 || y >= image.m_height - 1) )
						image.entry(x,y) = color;
					y++;
					D += 2*(-dx);
					if( D > 0 ){
						x--;
						D -= 2*dy;
					}
				}
			}

		}
	}


}