#pragma once
#include<cassert>
using namespace std;
#include<iostream>
using namespace std;

namespace dip{

template<class T>
class Block;

template<class T>
class plane{
public:
	plane();
	plane(const plane & rh);
	plane(int width, int height);

	~plane();


	void resize(int width, int height);
	void release();

	inline bool checkInside(int x, int y){return ( x >= 0 && y >= 0 && x < m_width && y < m_height);}
	inline bool checkInside(double x, double y){return ( x >= 0.0 && y >= 0.0 && x < static_cast<double>(m_width) && y < static_cast<double>(m_height) );}

	//access function
	inline T & entry(int x, int y){ return m_buf[y*m_width + x]; }
	inline const T & entry(int x, int y) const{ return m_buf[y*m_width + x]; }
	inline T & operator ()(int x, int y){ return m_buf[y*m_width + x]; }
	inline const T & operator()(int x, int y) const{ return m_buf[y*m_width + x]; }

	bool getBlock(Block<T> & b, int off_x, int off_y, int block_width, int block_height);

	plane<T> & operator=(const plane<T> & rh);
	inline bool operator==(const plane<T> & rh) const;

	plane<T> DownSample(int factor);
	
public:
	int m_width;
	int m_height;
	T* m_buf;

};

template<class T>
class Block{
public:
	Block();
	inline T & entry(int x, int y){
		int xx = x + m_off_x;
		int yy = y + m_off_y;
		return m_plane->entry(xx, yy);
	}

	inline const T & entry(int x, int y) const {
		int xx = x + m_off_x;
		int yy = y + m_off_y;
		return m_plane->entry(xx, yy);
	}

	int m_width;
	int m_height;

private:
	plane<T> * m_plane;
	int m_off_x;
	int m_off_y;
	friend class plane<T>;
};

/*************************************************************************************************
Implementation of plane
*************************************************************************************************/

template<class T>
plane<T>::plane() : m_buf(NULL) {
	m_buf = NULL;
	m_width = 0;
	m_height = 0;
}

template<class T>
plane<T>::plane(int width, int height) : m_buf(NULL){
	assert(width > 0 && height > 0);
	m_buf = new T[width*height];
	m_width = width;
	m_height = height;
	memset((char*)m_buf, 0, m_width*m_height*sizeof(T));
}

template<class T>
plane<T>::~plane(){
	if( m_buf != NULL)
		delete [] m_buf;
}


// copy constructor
template<class T>
plane<T>::plane( const plane<T> & rh) : m_buf(NULL) {
	if( rh.m_buf != NULL){
		m_width = rh.m_width;
		m_height = rh.m_height;
		m_buf = new T[m_width*m_height];
		memmove((void*)m_buf, (void*)rh.m_buf, m_width*m_height*sizeof(T));
	}else{
		m_buf = NULL;
		m_width = 0;
		m_height = 0;
	}
}

template<class T>
void plane<T>::resize(int width , int height){
	assert(width > 0 && height > 0);

	if( width*height == m_width*m_height ){
		m_width = width;
		m_height = height;
		return;
	}

	if( m_buf != NULL )
		delete [] m_buf;
	
	m_width = width;
	m_height = height;

	m_buf = new T[m_width*m_height];
	memset((char*)m_buf, 0, m_width*m_height*sizeof(T));
}

template<class T>
bool plane<T>::getBlock(Block<T> & b, int off_x, int off_y, int block_width, int block_height){
	if(block_width < 1 || block_height < 1)
		return false;

	if( off_x < 0 || off_y < 0 || off_x + block_width > m_width || off_y + block_height > m_height)
		return false;

	b.m_off_x = off_x;
	b.m_off_y = off_y;
	b.m_plane = this;
	b.m_height =  block_height;
	b.m_width = block_width;
	return true;
}

template<class T>
plane<T> plane<T>::DownSample(int factor){
	Image<T> result(m_width/factor, m_height/factor);
	for(int i = 0; i < result.m_width; i++){
		for(int j = 0; j < result.m_height; j++){
			result.entry(i,j) = this->entry(factor*i, factor*j);
		}
	}
	return result;
}

template<class T>
plane<T> & plane<T>::operator=(const plane<T> & rh){
	resize(rh.m_width, rh.m_height);
	memmove((void*)m_buf, (void*)rh.m_buf, m_width*m_height*sizeof(T));
	return *this;

}

template<class T>
inline bool plane<T>::operator ==(const plane<T> & rh) const{
	if(m_buf == NULL)
		return false;
	
	if( m_width == rh.m_width && m_height == rh.m_height )
		return true;
	else
		return false;
}

template<class T>
void plane<T>::release(){
	if( m_buf != NULL ){
		delete [] m_buf;
		m_buf = NULL;
	}
	m_width = 0;
	m_height = 0;
}

/*************************************************************************************************
Implementation of block
*************************************************************************************************/
template<class T>
Block<T>::Block(){
	int m_width = 0;
	int m_height = 0;
	plane<T> * m_plane = NULL;
	int m_off_x = 0;
	int m_off_y = 0;
}




} // end of namespace dip