#pragma once
#include "math_matrix.h"
using namespace math;

namespace math{

template<class T>
class Vector2{
public:
	Vector2();
	Vector2(T ix, T iy);
	T x;
	T y;

	Vector2<T> & operator=(const Vector2<T> & rh);
	Vector2<T> & operator+=(const Vector2<T> & rh);
	Vector2<T> & operator-=(const Vector2<T> & rh);

	Vector2<T> operator+(const Vector2<T> & rh) const;
	Vector2<T> operator-(const Vector2<T> & rh) const;

	template<class U>
	Vector2<T> operator*(U rh) const;

	template<class U>
	Vector2<T> operator/(U rh) const;

	Vector2<T> & operator*=(double scale);
	Vector2<T> & operator*=(const Matrix & rh);
	

	template<class T1>
	friend Vector2<T1> operator*(double scale, const Vector2<T1> & rh);
	template<class T2>
	friend Vector2<T2> operator*(const Matrix & lh, const Vector2<T2> & rh);

};


template<class T>
class Vector3{ 
public:
	Vector3();
	Vector3(T ix, T iy, T iz);
	T x;
	T y;
	T z;

	Vector3<T> & operator=(const Vector3<T> & rh);
	Vector3<T> & operator+=(const Vector3<T> & rh);
	Vector3<T> & operator-=(const Vector3<T> & rh);

	Vector3<T> operator+(const Vector3<T> & rh) const;
	Vector3<T> operator-(const Vector3<T> & rh) const;

	Vector3<T> & operator*=(double scale);
	Vector3<T> & operator*=(const Matrix & rh);
	template<class T1>
	friend Vector3<T1> operator*(double scale, const Vector3<T1> & rh);
	template<class T2>
	friend Vector3<T2> operator*(const Matrix & lh, const Vector3<T2> & rh);
	
};

typedef Vector2<int>    Vector2i;
typedef Vector2<double> Vector2d;
typedef Vector2<float>  Vector2f;

typedef Vector3<int>    Vector3i;
typedef Vector3<double> Vector3d;
typedef Vector3<float>  Vector3f;


/*************************************************************************************************
implementation of Vector2;
*************************************************************************************************/
template<class T>
Vector2<T>::Vector2() : x(0), y(0){
}

template<class T>
Vector2<T>::Vector2(T ix, T iy) : x(ix), y(iy){
}

template<class T>
Vector2<T> & Vector2<T>::operator=(const Vector2<T> & rh){
	x = rh.x;
	y = rh.y;
	return *this;
}

template<class T>
Vector2<T> & Vector2<T>::operator+=(const Vector2<T> & rh){
	x += rh.x;
	y += rh.y;
	return *this;
}

template<class T>
Vector2<T> & Vector2<T>::operator-=(const Vector2<T> & rh){
	x -= rh.x;
	y -= rh.y;
	return *this;
}

template<class T>
Vector2<T> Vector2<T>::operator+(const Vector2<T> & rh) const{
	Vector2<T> result;
	result.x = x + rh.x;
	result.y = y + rh.y;
	return result;
}

template<class T>
Vector2<T> Vector2<T>::operator-(const Vector2<T> & rh) const{
	Vector2<T> result;
	result.x = x - rh.x;
	result.y = y - rh.y;
	return result;
}

template<class T>
template<class U>
Vector2<T> Vector2<T>::operator*(U rh) const{
	Vector2<T> result;
	result.x = static_cast<T>(x * rh);
	result.y = static_cast<T>(y * rh);
	return result;
}

template<class T>
template<class U>
Vector2<T> Vector2<T>::operator/(U rh) const{
	Vector2<T> result;
	result.x = static_cast<T>(x / rh);
	result.y = static_cast<T>(y / rh);
	return result;
}




template<class T>
Vector2<T> & Vector2<T>::operator*=(double scale){
	x *= scale;
	y *= scale;
	return *this;
}

template<class T>
Vector2<T> & Vector2<T>::operator*=(const Matrix & rh){
	assert(rh.m_row == 2 && rh.m_col == 2);
	T x_ = rh.entry(0,0)*x + rh.entry(0,1)*y;
	T y_ = rh.entry(1,0)*x + rh.entry(1,1)*y;

	x = x_;
	y = y_;
	return *this;
}

template<class T1>
Vector2<T1> operator*(double scale, const Vector2<T1> & rh){
	Vector2<T1> result;
	result.x = scale*rh.x;
	result.y = scale*rh.y;
	return result;
}

template<class T2>
Vector2<T2> operator*(const Matrix & lh, const Vector2<T2> & rh){
	Vector2<T> result;
	result.x = lh.entry(0,0)*rh.x + lh.entry(0,1)*rh.y;
	result.y = lh.entry(1,0)*rh.x + lh.entry(1,1)*rh.y;
	return result;
}


/*************************************************************************************************
implementation of Vector3;
*************************************************************************************************/
template<class T>
Vector3<T>::Vector3(){
	x = 0;
	y = 0;
	z = 0;
}

template<class T>
Vector3<T>::Vector3(T ix, T iy, T iz){
	x = ix;
	y = iy;
	z = iz;
}

template<class T>
Vector3<T> & Vector3<T>::operator=(const Vector3<T> & rh){
	x = rh.x;
	y = rh.y;
	z = rh.z;
	return *this;
}

template<class T>
Vector3<T> & Vector3<T>::operator+=(const Vector3<T> & rh){
	x += rh.x;
	y += rh.y;
	z += rh.z;
	return *this;
}

template<class T>
Vector3<T> & Vector3<T>::operator-=(const Vector3<T> & rh){
	x -= rh.x;
	y -= rh.y;
	z -= rh.z;
	return *this;
}

template<class T>
Vector3<T> Vector3<T>::operator+(const Vector3<T> & rh) const{
	Vector3<T> result;
	result.x = x + rh.x;
	result.y = y + rh.y;
	result.z = z + rh.z;
	return result;
}

template<class T>
Vector3<T> Vector3<T>::operator-(const Vector3<T> & rh) const{
	Vector3<T> result;
	result.x = x - rh.x;
	result.y = y - rh.y;
	result.z = z - rh.z;
	return result;
}



template<class T>
Vector3<T> & Vector3<T>::operator*=(double scale){
	x *= scale;
	y *= scale;
	z *= scale;
	return *this;
}

template<class T>
Vector3<T> & Vector3<T>::operator*=(const Matrix & rh){
	assert(rh.m_row == 2 && rh.m_col == 2);
	T x_ = rh.entry(0,0)*x + rh.entry(0,1)*y + rh.entry(0,2)*z;
	T y_ = rh.entry(1,0)*x + rh.entry(1,1)*y + rh.entry(1,2)*z;
	T z_ = rh.entry(2,0)*x + rh.entry(2,1)*y + rh.entry(2,2)*z;

	x = x_;
	y = y_;
	z = z_;
	return *this;
}

template<class T1>
math::Vector3<T1> operator*(double scale, const Vector3<T1> & rh){
	Vector3<T> result;
	result.x = scale*rh.x;
	result.y = scale*rh.y;
	result.z = scale*rh.z;
	return result;
}

template<class T2>
math::Vector3<T2> operator*(math::Matrix const & lh, math::Vector3<T2> const & rh){
	Vector3<T2> result;
	result.x = lh.entry(0,0)*rh.x + lh.entry(0,1)*rh.y + lh.entry(0,2)*rh.z;
	result.y = lh.entry(1,0)*rh.x + lh.entry(1,1)*rh.y + lh.entry(1,2)*rh.z;
	result.z = lh.entry(2,0)*rh.x + lh.entry(2,1)*rh.y + lh.entry(2,2)*rh.z;
	return result;
}

} //end of namepsace math

