#include "IME_BRDF_Engine.h"
#include "ImgMatEdit.h"

bool BRDF_Engine::apply_brdf(float view_distance)
{
	g_ImgMatEdit.m_do_brdf = false;
	if(g_ImgMatEdit.m_do_inpaint == false)
		return false;
	if(g_ImgMatEdit.m_do_shape == false)
		return false;
	int x;
	int y;
	int i, j;
	int count;

	float w0 = float(rho_d / PI);
	float w1 = float(rho_s / (4*PI*rms));
	float w2;
	g_ImgMatEdit.m_brdf_img.resize(g_ImgMatEdit.m_srcimg.m_width, g_ImgMatEdit.m_srcimg.m_height);

	i = 0;

	//FILE *debug = fopen("ReBRDF.txt", "w");
	for(y = 0; y < g_ImgMatEdit.m_srcimg.m_height; y++)
	{
		for(x = 0; x < g_ImgMatEdit.m_srcimg.m_width; x++)
		{
			if(g_ImgMatEdit.m_alphaimg.entry(x,y).R == 0)
			{
				fRGB & C = g_ImgMatEdit.m_srcimg.m_buf[i];				
				g_ImgMatEdit.m_brdf_img.m_buf[i] = C;
			}
			else
			{
				// Apply BRDF evalutaion on the point (x,y);				
				V.set(x-xc, y-yc, view_distance);						// 1.  Calculate V
				N.set(	g_ImgMatEdit.m_normal_img.entry(x,y).R,			// 2.  Set N
						g_ImgMatEdit.m_normal_img.entry(x,y).G, 
						g_ImgMatEdit.m_normal_img.entry(x,y).B);
				
				cross_product(&N, &V, &a);								// 3.  Set a = N cross V
				N.normalize(); 
				V.normalize();
				a.normalize();
				alpha = dot(&N, &V);									// 4.  Set alpha = N dot V								
				r.set(alpha, &a);										// 5.  Set the rotation matrix by quaternion
				r_inv.inverse(&r);										// 5.2 Calculate the inverse matrix
				va.rotate(&r, &V);										// 6.  Calculate va
				cos_theta_v = va.z;
				
				// Calculate Equation (20)
				g_ImgMatEdit.m_brdf_img.entry(x,y).R = 0;
				g_ImgMatEdit.m_brdf_img.entry(x,y).G = 0;
				g_ImgMatEdit.m_brdf_img.entry(x,y).B = 0;
				count = 0;
				for(j = 0; j < brdf_samples; j++)
				{					
					D.set(d[j].x, d[j].y, d[j].z);					// 7. Get D
					da.rotate(&r, &D);				// 8. Calculate da
					cos_theta_d = da.z;
					theta_d = acos(cos_theta_d);

					if(cos_theta_d <= 0)
						continue;					

					h.middle(&va, &da);				// 9. Set h
					delta	= acos(h.z);
										
					// 10. Evaluate brdf using Ward's Model, (1/PI, 1/4*PI can be normalized to 2, 1/2)
					w2 = cos_theta_d * sin(theta_d);
					brdf_r = float(w0*g_r + w1/sqrt(cos_theta_d *cos_theta_v) * exp(-1*tan(delta)*tan(delta)/rms) );
					brdf_r = float(brdf_r * w2);
					brdf_g = float(w0*g_g + w1/sqrt(cos_theta_d *cos_theta_v) * exp(-1*tan(delta)*tan(delta)/rms) );
					brdf_g = float(brdf_g * w2);
					brdf_b = float(w0*g_b + w1/sqrt(cos_theta_d *cos_theta_v) * exp(-1*tan(delta)*tan(delta)/rms) );
					brdf_b = float(brdf_b * w2);

					// 11. Accumulate the intensity
					g_ImgMatEdit.m_brdf_img.entry(x,y).R += brdf_r * l.entry(j,0).R;
					g_ImgMatEdit.m_brdf_img.entry(x,y).G += brdf_g * l.entry(j,0).G;
					g_ImgMatEdit.m_brdf_img.entry(x,y).B += brdf_b * l.entry(j,0).B;
					count++;
				}
				// 12. Normalize
				//g_ImgMatEdit.m_brdf_img.entry(x,y).R;
				//g_ImgMatEdit.m_brdf_img.entry(x,y).G;
				//g_ImgMatEdit.m_brdf_img.entry(x,y).B;
			}
			//fprintf(debug, "%15.5f", g_ImgMatEdit.m_brdf_img.entry(x,y).G);
			i++;
		}
		//fprintf(debug, "\n");
	}
	//fclose(debug);
	g_ImgMatEdit.m_do_brdf = true;
	return true;
}

void BRDF_Engine::make_samples(FloatRgbImage *env_map)
{
	int i;
	float x, y, side;
	int x0, y0;
	float xd0, xd1;
	float yd0, yd1;
	float w0, w1, w2, w3;
	xc = float(env_map->m_width/2.0);
	yc = float(env_map->m_height/2.0);
	R = min(xc, yc)-1;

	//FILE *debug = fopen("Samples.txt", "w");
	for(i = 0; i < brdf_samples; i++)
	{
		d[i].x = float(rand())/RAND_MAX;		d[i].x = float(d[i].x - 0.5)*2;
		d[i].y = float(rand())/RAND_MAX;		d[i].y = float(d[i].y - 0.5)*2;
		side   = float(rand())/RAND_MAX;
		if(side > 0.5)
			d[i].z = 1 - sqrt(d->x * d->x + d->y * d->y);
		else
			d[i].z = -1 +sqrt(d->x * d->x + d->y * d->y);


		x = d[i].x * R + xc;
		y = d[i].y * R + yc;

		d[i].normalize();
		// Sample the environment
		x0 = (int)floor(x);	xd0 = x-x0;	xd1 = 1 - xd0;
		y0 = (int)floor(y);	yd0 = y-y0; yd1 = 1 - yd0;
		w0 = xd1 * yd1;
		w1 = xd1 * yd0;
		w2 = xd0 * yd1;
		w3 = xd0 * yd0;
		l.entry(i, 0).R =	env_map->entry(x0  , y0  ).R * w0 +
							env_map->entry(x0  , y0+1).R * w1 + 
							env_map->entry(x0+1, y0  ).R * w2 + 
							env_map->entry(x0+1, y0+1).R * w3;
		l.entry(i, 0).G =	env_map->entry(x0  , y0  ).G * w0 +
							env_map->entry(x0  , y0+1).G * w1 + 
							env_map->entry(x0+1, y0  ).G * w2 + 
							env_map->entry(x0+1, y0+1).G * w3;
		l.entry(i, 0).B =	env_map->entry(x0  , y0  ).B * w0 +
							env_map->entry(x0  , y0+1).B * w1 + 
							env_map->entry(x0+1, y0  ).B * w2 + 
							env_map->entry(x0+1, y0+1).B * w3;
		//fprintf(debug, "%15.5f %15.5f %15.5f %15.5f %15.5f %15.5f\n", d[i].x, d[i].y, d[i].z, l.entry(i, 0).R, l.entry(i, 0).G, l.entry(i, 0).B);
	}
	//fclose(debug);
}

void BRDF_Engine::set_parameters(float rho_d_in, float rho_s_in, float rms_in)
{
	rho_d = rho_d_in;
	rho_s = rho_s_in;
	rms = rms_in;
}

void BRDF_Engine::set_color(float r, float g, float b)
{
	g_r = r;
	g_g = g;
	g_b = b;
}

void BRDF_Engine::make_preview()
{
	FloatRgbImage brdf_tonemap;
	g_ImgMatEdit.ToneMapping(g_ImgMatEdit.m_brdf_img, brdf_tonemap);
	g_ImgMatEdit.m_brdf_previmg = ToNormalImage(brdf_tonemap);
}

BRDF_Engine::BRDF_Engine()
{
	d = new vector3 [brdf_samples];
	l.resize(brdf_samples,1);
	time_t seconds = time(NULL);
	srand((unsigned int)seconds);

	rho_d = (float)0.29;
	rho_s = (float)0.083;
	rms   = (float)0.082;

	g_r   = 1.0;
	g_g	  = 1.0;
	g_b	  = 1.0;
}

BRDF_Engine::~BRDF_Engine()
{
	delete []d;
}