#pragma once
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <ctime>

using namespace std;

#include "stdafx.h"
#include "ProjMgr.h"
#include "matrix.h"

#define PI		(3.141592653589793)
#define TWO_PI	PI*2

static int brdf_samples = 200;

class BRDF_Engine
{
public:
	BRDF_Engine();
	~BRDF_Engine();

	void make_samples(FloatRgbImage *env_map);
	void set_parameters(float rho_d_in, float rho_s_in, float rms_in);
	void set_color(float r, float g, float b);
	bool apply_brdf(float view_distance);
	void make_preview();

	// Ward's isotropic model
	float rho_d;
	float rho_s;
	float rms;

	// For anisotripic model
	float rms_x;
	float rms_y;

	// For chromatic values
	float g_r;
	float g_g;
	float g_b;

	// Samples for BRDF evaluation

	vector3			*d;
	FloatRgbImage	l;

	float R;
	float xc;
	float yc;

	float alpha;

	// Vectors in global coordinate;
	vector3 V;
	vector3 D;
	vector3 N;	//Object normal;
	vector3 a;

	// Vectors in local coordinate;
	vector3 da;
	vector3 va;
	vector3 h;
	vector3 q;

	// Matrix for rotation
	matrix3 r;
	matrix3 r_inv;

	// Temporary variables
	float theta_v;
	float theta_d;
	float cos_theta_v;
	float cos_theta_d;
	float delta;

	float brdf_r;
	float brdf_g;
	float brdf_b;
};