#include "StdAfx.h"
#include ".\imgmatedit.h"
using namespace std;

GROSS_SPEC_INFO::GROSS_SPEC_INFO(){
	alpha = 0.05f;
	beta = 0.05f;
	is_gross = true;
	use_histogram = true;
	user_def_lmin = -1.0f;
}

bool ImgMatEdit::GrossSpecular(){

	if( m_alphaimg.m_width == 0 || m_alphaimg.m_height == 0)
		return false;

	int size = m_srcimg.m_width*m_srcimg.m_height;
	m_gross_intensity.resize(m_srcimg.m_width, m_srcimg.m_height);

	// if we need histogram and the histogram has not been constructed, then build the histogram.
	// if failed to build histogram, return false.
	if( ! m_do_histogram ){
		if( !ImgHistogram() )
			return false;
	}

	float Lmin;

	if( m_gross_spec_info.use_histogram )
		Lmin = m_his_info.hl_threshold;
	else
		Lmin = m_gross_spec_info.user_def_lmin;

	// start algorithm
	// Equation (18), specular
	if( !m_gross_spec_info.is_gross ){
		for(int i = 0; i < size; i++){
			if( m_alphaimg.m_buf[i].R == 0)
				continue;
			float & L = m_intensity.m_buf[i];

			if( L > Lmin ){
				m_gross_intensity.m_buf[i] = Lmin + (m_his_info.lmax - Lmin)*pow((m_gross_spec_info.alpha*(L - Lmin)/(m_his_info.lmax - Lmin)), m_gross_spec_info.beta);
			}
			else if( L <= Lmin){
				m_gross_intensity.m_buf[i] = L;
			}
		}
	}
	// Equation (19), gross
	else{
		//m_gross_intensity = m_intensity;
		for(int i = 0; i < size; i++){
			if( m_alphaimg.m_buf[i].R == 0)
				continue;
			float & L = m_intensity.m_buf[i];

			if( L > m_his_info.his_peak ){
				m_gross_intensity.m_buf[i] = m_his_info.his_peak + (Lmin - m_his_info.his_peak)*pow( (L - m_his_info.his_peak)/(m_his_info.lmax - m_his_info.his_peak), m_gross_spec_info.beta);
			}else{
				m_gross_intensity.m_buf[i] = L;
			}

		}
	}
	
	FloatRgbImage gross_outimg( m_srcimg.m_width, m_srcimg.m_height);
	
	// adjust the color according to the new luminance
	for(int i = 0; i < size; i++){
		fRGB & C = m_srcimg.m_buf[i];
		if( m_alphaimg.m_buf[i].R == 0 ){
			gross_outimg.m_buf[i] = C;
		}else{
			gross_outimg.m_buf[i].R = C.R * m_gross_intensity.m_buf[i] / ( m_intensity.m_buf[i] + 1e-7f );
			gross_outimg.m_buf[i].G = C.G * m_gross_intensity.m_buf[i] / ( m_intensity.m_buf[i] + 1e-7f );
			gross_outimg.m_buf[i].B = C.B * m_gross_intensity.m_buf[i] / ( m_intensity.m_buf[i] + 1e-7f );
		}
	}

	FloatRgbImage gross_tonemap;
	ToneMapping(gross_outimg, gross_tonemap);

	m_gross_previmg = ToNormalImage( gross_tonemap);
	m_do_gross_spec = true;
	
	return true;
}