#include "StdAfx.h"
#include ".\imgmatedit.h"
#include <exception>
using namespace std;


bool ImgMatEdit::FastToneMapping(FloatRgbImage & input, FloatRgbImage & output, float a, float Lwhite){
	FloatImage intensity = ToIntensity(input);
	FloatImage logimg(input.m_width, input.m_height);
	int imgsize = input.m_width * input.m_height;

	float Lw = 0.0f;
	float log10_inv = 1.0f/log(10.0f);
	for(int i = 0; i < imgsize ; i++){
		logimg.m_buf[i] = log(intensity.m_buf[i] + 1e-6f)*log10_inv;
		Lw += logimg.m_buf[i];
	}

	Lw /= imgsize;
	Lw = pow(10.0f, Lw);

	FloatImage &tonemap = m_tonemapped_intensity;
	tonemap.resize(input.m_width, input.m_height);
	float Lw_inv_a = (1.0f/Lw)*a;
	float Lwhite_inv_square = 1.0f/(Lwhite*Lwhite);
	for(int i = 0; i < imgsize; i++){
		tonemap.m_buf[i] = intensity.m_buf[i]*Lw_inv_a;
		tonemap.m_buf[i] = tonemap.m_buf[i]/(1.0f + tonemap.m_buf[i]);
		tonemap.m_buf[i] = tonemap.m_buf[i]*(1 + tonemap.m_buf[i]*Lwhite_inv_square)/(1 + tonemap.m_buf[i]);
	}

	output.resize(input.m_width, input.m_height);	
	for(int i = 0; i < imgsize; i++){
		float rate = tonemap.m_buf[i] / (intensity.m_buf[i] + 1e-6f);
		output.m_buf[i].R = max(min(input.m_buf[i].R * rate, 1.0f), 0.0f);
		output.m_buf[i].G = max(min(input.m_buf[i].G * rate, 1.0f), 0.0f);
		output.m_buf[i].B = max(min(input.m_buf[i].B * rate, 1.0f), 0.0f);
		
		// gamma correction
		output.m_buf[i].R = pow(output.m_buf[i].R, 1.0f/2.2f);
		output.m_buf[i].G = pow(output.m_buf[i].G, 1.0f/2.2f);
		output.m_buf[i].B = pow(output.m_buf[i].B, 1.0f/2.2f);
	}

	return true;
}

bool ImgMatEdit::SlowToneMapping(FloatRgbImage & input, FloatRgbImage & output, float a, float Lwhite){
	FloatImage intensity = ToIntensity(input);
	FloatImage logimg(input.m_width, input.m_height);
	int imgsize = input.m_width * input.m_height;

	float Lw = 0.0f;
	float log10_inv = 1.0f/log(10.0f);
	for(int i = 0; i < imgsize ; i++){
		logimg.m_buf[i] = log(intensity.m_buf[i] + 1e-6f)*log10_inv;
		Lw += logimg.m_buf[i];
	}

	Lw /= imgsize;
	Lw = pow(10.0f, Lw);

	FloatImage detial( input.m_width, input.m_height);
	FloatImage base(input.m_width, input.m_height);

	FloatImage space_gaussian = GaussianFilter2D(5.0f);


	//biliteral filtering - get basse
	for(int y = 0; y < logimg.m_height; y++){
		for(int x = 0; x < logimg.m_width; x++){
			static int center_x = space_gaussian.m_width/2;
			static int center_y = space_gaussian.m_height/2;
			float v = 0.0f;
			float m = 0.0f;
			for(int by = -space_gaussian.m_height/2 ; by <= space_gaussian.m_height/2; by++){
				for(int bx = -space_gaussian.m_width/2; bx <= space_gaussian.m_width/2; bx++){
					int xx = bx + x;
					int yy = by + y;
					if( logimg.checkInside( xx, yy) ){
						float diff = fabs(logimg.entry( xx, yy ) - logimg.entry(x,y));
						float w = space_gaussian.entry(center_x + bx, center_y + by) * Gaussian(diff, 0.4f);
						v += logimg.entry(xx, yy) * w;
						m += w;
					}
				}
			}

			if( m != 0 )
				v /= m;

			base.entry( x, y ) = v;
			//base.entry( x,y )
		}
	}

	//get detial

	for(int i = 0 ; i < imgsize; i++){
		detial.m_buf[i] = logimg.m_buf[i] - base.m_buf[i];
	}

	//recover base from log domain
	for(int i = 0 ; i < imgsize; i++){
		base.m_buf[i] = pow(10.0f, base.m_buf[i]);
	}


	// tonemap base 
	FloatImage &tonemap = m_tonemapped_intensity;
	tonemap.resize(input.m_width, input.m_height);
	float Lw_inv_a = (1.0f/Lw)*a;
	float Lwhite_inv_square = 1.0f/(Lwhite*Lwhite);

	for(int i = 0; i < imgsize; i++){
		tonemap.m_buf[i] = base.m_buf[i]*Lw_inv_a;
		tonemap.m_buf[i] = tonemap.m_buf[i]/(1.0f + tonemap.m_buf[i]);
		tonemap.m_buf[i] = tonemap.m_buf[i]*(1 + tonemap.m_buf[i]*Lwhite_inv_square)/(1 + tonemap.m_buf[i]);
	}


	// recover detail from log domain
	for(int i = 0; i < imgsize; i++){
		detial.m_buf[i] = pow(10.0f, detial.m_buf[i]);
	}

	// combine tonemap

	for(int i = 0; i < imgsize; i++){
		tonemap.m_buf[i] = tonemap.m_buf[i]*detial.m_buf[i];
	}


	output.resize(input.m_width, input.m_height);
	
	for(int i = 0; i < imgsize; i++){
		float rate = tonemap.m_buf[i] / (intensity.m_buf[i] + 1e-6f);
		output.m_buf[i].R = max(min(input.m_buf[i].R * rate, 1.0f), 0.0f);
		output.m_buf[i].G = max(min(input.m_buf[i].G * rate, 1.0f), 0.0f);
		output.m_buf[i].B = max(min(input.m_buf[i].B * rate, 1.0f), 0.0f);
		
		// gamma correction
		output.m_buf[i].R = pow(output.m_buf[i].R, 1.0f/2.2f);
		output.m_buf[i].G = pow(output.m_buf[i].G, 1.0f/2.2f);
		output.m_buf[i].B = pow(output.m_buf[i].B, 1.0f/2.2f);
	}

	return true;

}