#include "StdAfx.h"
#include ".\imgmatedit.h"
#include "hdrloader.h"
#include <fstream>
#include <iomanip>
#include <cstdio>
#include "rgbe.h"

using namespace std;

ImgMatEdit g_ImgMatEdit;

ImgMatEdit::ImgMatEdit(void)
{
	ResetStatus();
}

ImgMatEdit::~ImgMatEdit(void)
{
}


void ImgMatEdit::ResetStatus(){
	m_slow_tonemapping = false;
	m_do_histogram = false;
	m_do_gross_spec = false;
	m_do_hightlights = false;
	m_do_shape = false;
	m_do_brdf = false;
	m_use_gross = false;


	m_g_value = 3.0f;

	m_gross_previmg.release();
	m_highlights_img.release();
	m_depth_previmg.release();
	m_inpaint_previmg.release();
	m_retex_previmg.release();
	m_alphaimg.release();
}

bool ImgMatEdit::LoadSrcImg(char *filename){
	//need modify since our input is hdr

	try{
		//HDRLoader::load(filename, m_srcimg);
		//m_outimg = ReadBmpFile(filename);
		int image_width = 0, image_height = 0;
		FILE* f = fopen(filename, "rb");
		RGBE_ReadHeader(f, &image_width, &image_height,NULL);

		m_srcimg.resize( image_width, image_height);
	
		//image = (float *)malloc(sizeof(float)*3*image_width*image_height);
		RGBE_ReadPixels_RLE(f,(float*)m_srcimg.m_buf,image_width,image_height);

	}
	catch(exception e){
		return false;
	}

	ResetStatus();

	if( m_alphaimg.m_width != 0 && m_alphaimg.m_height != 0){
		ImgHistogram();
	}

	m_intensity = ToIntensity(m_srcimg);

	PreviewSource();
	//m_srcimg = ToFloatImage(m_outimg);
	return true;
}

bool ImgMatEdit::LoadAlphaImg(char *filename){
	ResetStatus();
	try{
		m_alphaimg = ReadBmpFile(filename);
	}
	catch(exception e){
		return false;
	}


	if( m_srcimg.m_width != 0 && m_srcimg.m_height != 0){
		ImgHistogram();
	}

	return true;
}


bool ImgMatEdit::PreviewSource(){
	FloatRgbImage tonemap;
	ToneMapping(m_srcimg, tonemap);
	m_src_previmg = ToNormalImage(tonemap);
	return true;
}

bool ImgMatEdit::ImgHistogram(){
	if( m_srcimg.m_width == 0 || m_srcimg.m_height == 0)
		return false;

	if( m_alphaimg.m_width == 0 || m_alphaimg.m_height == 0)
		return false;

	if( m_intensity.m_width == 0 || m_intensity.m_height == 0)
		return false;

	for(int i = 0 ; i < 71; i++){
		m_histogram[i] = 0;
		m_derivative[i] = 0;
	}

	int size = m_srcimg.m_width * m_srcimg.m_height;

	
	// find Max and Min
	float pel_max = 0.0f;
	float pel_min = 10000.0f;

//	m_hisLmax = 0.0f;
//	m_his_base = 10000.0f;
	for(int i = 0; i < size; i++){

		if( m_alphaimg.m_buf[i].R == 0)
			continue;

		if( m_intensity.m_buf[i] > pel_max)
			pel_max =  m_intensity.m_buf[i];

		if( m_intensity.m_buf[i] < pel_min){
			pel_min = m_intensity.m_buf[i];
		}
	}

	m_his_info.lmax = pel_max;
	m_his_info.lmin = pel_min;
	m_his_info.his_base = log(max(0.001f, pel_min))/log(10.0f);
	m_his_info.bin_space = ((log(pel_max)/log(10.0f)) - m_his_info.his_base)/70;

	// histogram image pixels
	for(int i = 0; i < size; i++){

		if( m_alphaimg.m_buf[i].R == 0)
			continue;

		if( m_intensity.m_buf[i] < 0.001f )
			continue;

		float L = log(m_intensity.m_buf[i]) / log(10.0f);

		int bin = int_round((L - m_his_info.his_base)/m_his_info.bin_space);
		if( bin < 71 && bin >= 0){
			m_histogram[bin]++;
		}
	}

	// calculate derivative

	for(int i = 1; i < 70; i++){
		m_derivative[i] = (m_histogram[i + 1] - m_histogram[i - 1])/2.0f;
	}


	// find default Lmin
	float hl_threshold = 0;
	float m = 1e7f;

	for(int i = 1; i < 70; i++){
		if( m > m_derivative[i]){
			m = m_derivative[i];
			hl_threshold = (float)i;
		}
	}

	m_his_info.hl_threshold = pow(10.0f, hl_threshold*m_his_info.bin_space + m_his_info.his_base);

	// find Lhmax

	int M = 0;
	float his_max = 0;
	float chart_unit = 0;

	for(int i = 0; i < 71; i++){
		if( m_histogram[i] > M){
			M = m_histogram[i];
			his_max = (float)i;
		}
	}

	m_his_info.his_peak = pow(10.0f, his_max*m_his_info.bin_space + m_his_info.his_base);

	chart_unit = M /99.0f;

	m_do_histogram = true;

	// draw histogram
	const int w = 3;
	m_histogram_chart.resize(71*w, 100);

	for(int x = 0; x < m_histogram_chart.m_width; x++){
		for(int y = 0; y < m_histogram_chart.m_height; y++){
			m_histogram_chart.entry(x,y) = uRGB(0xCC, 0xCC, 0xCC);
		}
	}
	

	for(int x = 0; x < 71; x++){
		for(int cx = 0; cx < w ; cx++){
			for(int y = 0; y < m_histogram[x]/chart_unit; y++){
				m_histogram_chart.entry(x*w + cx, y) = uRGB(0, 0, 255);
			}
		}
	}

	//--------------- dump histogram -----------------------
	ofstream file("histogram.txt", ios::trunc);

	file << setw(20) << pow(10.0f, m_his_info.his_base) << setw(20) << m_histogram[0] << endl;

	for(int i = 1; i < 70; i++){

		file << setw(20) << pow(10.0f, m_his_info.his_base + i*m_his_info.bin_space) << setw(20) << m_histogram[i] << setw(20) << m_derivative[i] << endl;
	}

	file << setw(20) << pow(10.0f, m_his_info.his_base + 70*m_his_info.bin_space) << setw(20) << m_histogram[70] << endl;

	file.close();

	return true;
}

bool ImgMatEdit::DrawHistogram(RgbImage & out){
	if( !m_do_histogram )
		return false;

	out.resize(210, 150);
	int size = 210*150;
	for(int i = 0; i < size; i++){
		out.m_buf[i] = uRGB(0xCC, 0xCC, 0xCC);
	}

	for(int i = 0; i < 210; i++){
		out.entry(i, 149) = uRGB(0,0,0);
	}

	for(int i = 0; i < 150; i++){
		out.entry(0, i) = uRGB(0,0,0);
	}

	return true;

}

bool ImgMatEdit::DetectHeightLightsPel(float Lmin, bool useHis){

	if( useHis ){
		if( !m_do_histogram)
			if(!ImgHistogram())
				return false;

		m_highlight_lmin = m_his_info.hl_threshold;
	}else{
		m_highlight_lmin = Lmin;
	}

	m_do_hightlights = MarkHigherPel(Lmin, m_highlights_img);
	return m_do_hightlights;

	

	/*
	m_highlights_img.resize(m_srcimg.m_width, m_srcimg.m_height);


	int size = m_srcimg.m_width * m_srcimg.m_height;

	for(int i = 0; i < size; i++){
		if( m_alphaimg.m_buf[i].R == 0){
			m_highlights_img.m_buf[i] = uRGB(0, 0, 0);
			continue;
		}

		if( m_intensity.m_buf[i] >= m_highlight_lmin )
			m_highlights_img.m_buf[i] = uRGB(255, 255, 255);
		else
			m_highlights_img.m_buf[i] = uRGB(0, 0, 0);
	}

	return true;
	*/

}


bool ImgMatEdit::MarkHigherPel(float threshold, RgbImage & outmask){
	outmask.resize(m_srcimg.m_width, m_srcimg.m_height);
	int size = m_srcimg.m_width * m_srcimg.m_height;

	for(int i = 0; i < size; i++){
		if( m_alphaimg.m_buf[i].R == 0){
			outmask.m_buf[i] = uRGB(0, 0, 0);
			continue;
		}

		if( m_intensity.m_buf[i] >= threshold )
			outmask.m_buf[i] = uRGB(255, 255, 255);
		else
			outmask.m_buf[i] = uRGB(0, 0, 0);
	}

	return true;
}


bool ImgMatEdit::Inpaint(){

	int Xmin = m_srcimg.m_width;
	int Xmax = 0;

	//find Xmin and Xmax

	m_inpainted_img.resize( m_srcimg.m_width, m_srcimg.m_height);
	for(int y = 0; y < m_srcimg.m_height; y++){
		for(int x = 0; x < m_srcimg.m_width; x++){
			if( m_alphaimg.entry(x,y).R != 0 ){
				if( x < Xmin)
					Xmin = x;
				if( x > Xmax)
					Xmax = x;
			}
		}
	}

	for(int y = 0; y < m_srcimg.m_height; y++){
		for(int x = 0; x < m_srcimg.m_width; x++){
			if( m_alphaimg.entry(x,y).R != 0 ){
				double w1 = ( x - Xmin)/(double)(Xmax - Xmin);
				double w2 = ( Xmax - x)/(double)(Xmax - Xmin);
				m_inpainted_img.entry(x,y) = m_srcimg.entry(2*Xmin - x, y)*w1 + m_srcimg.entry(2*Xmax - x, y)*w2;
			}else{
				m_inpainted_img.entry(x,y) = m_srcimg.entry(x,y);
			}
		}
	}

	m_inpainted_blur_img = m_inpainted_img.ConvFast(GaussianFilter1D(m_g_value));

	m_do_inpaint = true;

	FloatRgbImage tonemap_inpainted;
	ToneMapping( m_inpainted_blur_img, tonemap_inpainted);
	m_inpaint_previmg = ToNormalImage(tonemap_inpainted);
	return true;

}

RETEX_INFO::RETEX_INFO(){
	scalar_s = 15.0f;
	scalar_f = 0.0;
	tex_size = 1.0f;
	scalar_tex = 1.0f;
	black_glass = false;
	preserve_hl = false;
	mode = tx_transparency;
	color = fRGB(1.0f, 1.0f, 1.0f);
}

bool ImgMatEdit::Retexture(){
	if( m_retex_info.mode == tx_transparency){
		return Retex_Transparent();
	}else
		return Retex_External();
}

bool ImgMatEdit::Retex_Transparent(){

	if( !m_do_shape ){
		return false;
	}

	plane<Vector2f> tex_coord(m_alphaimg.m_width, m_alphaimg.m_height);

	if( !m_do_inpaint)
		Inpaint();
	FloatRgbImage* T = &m_inpainted_blur_img;


	if( T->m_width == 0 || T->m_height == 0)
		return false;

	for(int y = 0; y < m_srcimg.m_height; y++){
		for(int x = 0; x < m_srcimg.m_width; x++){
			if( m_alphaimg.entry(x,y).R != 0){
				float tx = x + m_retex_info.scalar_s*m_gradient_img.entry(x, y).R;
				float ty = y + m_retex_info.scalar_s*m_gradient_img.entry(x, y).G;

				tx = fmod(tx, (float)T->m_width);
				ty = fmod(ty, (float)T->m_height);

				if( tx < 0)
					tx += T->m_width;

				if( ty < 0)
					ty += T->m_height;

				if( tx > T->m_width - 1){
					tx = (float)(T->m_width - 1);
				}

				if( ty > T->m_height - 1){
					ty = (float)(T->m_height - 1);
				}

				tex_coord.entry(x,y).x = (float)tx;
				tex_coord.entry(x,y).y = (float)ty;
			}
		}
	}

	float Cmax = 0;
	int size = m_srcimg.m_width * m_srcimg.m_height;
	m_retex_img.resize(m_srcimg.m_width, m_srcimg.m_height);

	fRGB  A;
	
	//if( mode == tx_transparency)
		A = fRGB( ( 1 - m_retex_info.scalar_f)*m_retex_info.color.R, ( 1 - m_retex_info.scalar_f)*m_retex_info.color.G, ( 1 - m_retex_info.scalar_f)*m_retex_info.color.B);
	//else
		//A = fRGB( ( 1 - scalar_f)*scalar_tex*color.R, ( 1 - scalar_f)*scalar_tex*color.G, ( 1 - scalar_f)*scalar_tex*color.B);

	float B = m_retex_info.scalar_f;

	for(int i = 0 ; i < size; i++){
		if( m_alphaimg.m_buf[i].R != 0){
			fRGB &tex  = T->lerp_entry( (double)tex_coord.m_buf[i].x, (double)tex_coord.m_buf[i].y);
			m_retex_img.m_buf[i].R = A.R*tex.R + B*m_srcimg.m_buf[i].R;
			m_retex_img.m_buf[i].G = A.G*tex.G + B*m_srcimg.m_buf[i].G;
			m_retex_img.m_buf[i].B = A.B*tex.B + B*m_srcimg.m_buf[i].B;

			if( m_retex_img.m_buf[i].R > Cmax ) {
				Cmax = m_retex_img.m_buf[i].R;
			}
			if( m_retex_img.m_buf[i].G > Cmax ) {
				Cmax = m_retex_img.m_buf[i].G;
			}

			if( m_retex_img.m_buf[i].B > Cmax ) {
				Cmax = m_retex_img.m_buf[i].B;
			}

		}else{
			m_retex_img.m_buf[i] = m_srcimg.m_buf[i];
		}
	}

	if( m_retex_info.black_glass ){
		float Cmax_inv = 1.0f/Cmax;
		for(int i = 0; i < size; i++){
			if( m_alphaimg.m_buf[i].R != 0 ){
				m_retex_img.m_buf[i].R = Cmax*pow(m_retex_img.m_buf[i].R  * Cmax_inv, 1.2f);
				m_retex_img.m_buf[i].G = Cmax*pow(m_retex_img.m_buf[i].G  * Cmax_inv, 1.2f);
				m_retex_img.m_buf[i].B = Cmax*pow(m_retex_img.m_buf[i].B  * Cmax_inv, 1.2f);
			}
		}
	}

	if( m_retex_info.preserve_hl && m_do_hightlights){
		for(int i = 0 ; i < size; i++){
			if( this->m_highlights_img.m_buf[i].R != 0 ){
				m_retex_img.m_buf[i] = m_srcimg.m_buf[i];
			}
		}
	}



	FloatRgbImage prev_img;
	ToneMapping(m_retex_img, prev_img);
	m_retex_previmg = ToNormalImage(prev_img);
	return true;
}

bool ImgMatEdit::Retex_External(){

	if( !m_do_shape ){
		return false;
	}

	plane<Vector2f> tex_coord(m_alphaimg.m_width, m_alphaimg.m_height);

	FloatRgbImage* T = &m_external_tex;

	if( T->m_width == 0 || T->m_height == 0)
		return false;

	for(int y = 0; y < m_srcimg.m_height; y++){
		for(int x = 0; x < m_srcimg.m_width; x++){
			if( m_alphaimg.entry(x,y).R != 0){
				float tx = x + m_retex_info.scalar_s*m_gradient_img.entry(x, y).R;
				float ty = y + m_retex_info.scalar_s*m_gradient_img.entry(x, y).G;

				tx = fmod(tx*m_retex_info.tex_size, (float)T->m_width);
				ty = fmod(ty*m_retex_info.tex_size, (float)T->m_height);

				if( tx < 0)
					tx += T->m_width;

				if( ty < 0)
					ty += T->m_height;

				if( tx > T->m_width - 1){
					tx = (float)(T->m_width - 1);
				}

				if( ty > T->m_height - 1){
					ty = (float)(T->m_height - 1);
				}

				tex_coord.entry(x,y).x = (float)tx;
				tex_coord.entry(x,y).y = (float)ty;
			}
		}
	}

	int size = m_srcimg.m_width * m_srcimg.m_height;
	m_retex_img.resize(m_srcimg.m_width, m_srcimg.m_height);


	ToneMapping(m_srcimg, m_retex_img);

	float scalar_tex = m_retex_info.scalar_tex * 5.0f;
	fRGB A = fRGB( ( 1 - m_retex_info.scalar_f)*scalar_tex*m_retex_info.color.R, ( 1 - m_retex_info.scalar_f)*scalar_tex*m_retex_info.color.G, ( 1 - m_retex_info.scalar_f)*scalar_tex*m_retex_info.color.B);
	m_retex_info.color*((1.0f - m_retex_info.scalar_f)*m_retex_info.scalar_tex);//
	float B = m_retex_info.scalar_f;

	for(int i = 0; i < size; i++){
		if( m_alphaimg.m_buf[i].R != 0){
			fRGB &tex  = T->lerp_entry( (double)tex_coord.m_buf[i].x, (double)tex_coord.m_buf[i].y);
			float I = m_tonemapped_intensity.m_buf[i];
			m_retex_img.m_buf[i].R = A.R*tex.R*I + B*m_retex_img.m_buf[i].R;
			m_retex_img.m_buf[i].G = A.G*tex.G*I + B*m_retex_img.m_buf[i].G;
			m_retex_img.m_buf[i].B = A.B*tex.B*I + B*m_retex_img.m_buf[i].B;
			
			m_retex_img.m_buf[i].R = max(min(m_retex_img.m_buf[i].R, 1.0f), 0.0f);
			m_retex_img.m_buf[i].G = max(min(m_retex_img.m_buf[i].G, 1.0f), 0.0f);
			m_retex_img.m_buf[i].B = max(min(m_retex_img.m_buf[i].B, 1.0f), 0.0f);
		}
	}
	m_retex_previmg = ToNormalImage(m_retex_img);
	return true;
}


bool ImgMatEdit::DoFinal(out_selection x){
	m_slow_tonemapping = true;
	bool res = false;
	switch(x){
		case os_source:
			res = PreviewSource();
			break;
		case os_retexture:
			res = Retexture();
			break;
		case os_brdf:
			brdf.make_preview();
			break;
		case os_gross:
			res = GrossSpecular();
			break;
		case os_inpaint:
			res = Inpaint();
			break;
	}

	m_slow_tonemapping = false;
	return res;
	
}