#pragma once
#include "stdafx.h"
#include "ProjMgr.h"
#include "ShapeEstimator.h"
#include "IME_BRDF_Engine.h"

enum tx_mode{tx_external, tx_transparency};

struct HISTOGRAM_INFO{
	float hl_threshold;
	float his_peak;
	float his_base;
	float lmax;
	float lmin;
	float bin_space;
};


struct RETEX_INFO{
	RETEX_INFO();
	float scalar_s;
	float scalar_f;
	float scalar_tex;
	float tex_size;
	fRGB  color;
	bool black_glass;
	bool preserve_hl;
	tx_mode mode;
};

struct GROSS_SPEC_INFO{
	GROSS_SPEC_INFO();

	float alpha;
	float beta;
	bool is_gross;
	bool use_histogram;
	float user_def_lmin;
};

enum out_selection{ os_retexture, os_brdf, os_gross, os_inpaint, os_source};

class ImgMatEdit
{
public:
	ImgMatEdit(void);
	~ImgMatEdit(void);

	bool LoadSrcImg( char* filename);
	bool LoadAlphaImg(char* filename);


	bool OutputRender();
	bool PreviewSource();
	bool DoFinal(out_selection x);

	void ResetStatus();

	bool MarkHigherPel(float threshold, RgbImage & outmask);

public:
	FloatRgbImage m_srcimg; //high dynamic rangle
	FloatImage m_intensity;
	FloatImage m_tonemapped_intensity;
	RgbImage m_alphaimg;
	RgbImage m_src_previmg;

	////////members of modules//////////////////////////////////////////////////////////////

	////////for histogram///////////////////
public:
	bool m_do_histogram;

	bool ImgHistogram();
	int m_histogram[71];
	float m_derivative[71];
private:
	HISTOGRAM_INFO m_his_info;
public:

	//general domin
	//float m_hisLmin;    // minimum of highlight
	//float m_hisLhmax;
	//float m_hisLmax;
	bool DrawHistogram(RgbImage & out);

	//log domain
	//float m_bin_space;
	//float m_his_base;   // minimum of the whole object

	inline const HISTOGRAM_INFO & GetHisInfo(){return m_his_info;}
	inline float LogToValue( float log_v ){ return pow(10.0f, log_v*m_his_info.bin_space + m_his_info.his_base); }
	inline float ValueToLog( float v){  if( v < 1e-3) return 0; else return ((log(v)/log(10.0f)) - m_his_info.his_base)/m_his_info.bin_space; }
	RgbImage m_histogram_chart;

	////////tone mapping///////////////////
public:
	bool FastToneMapping(FloatRgbImage & input, FloatRgbImage & output, float a = 0.27f, float Lwhite = 3.0f);
	bool SlowToneMapping(FloatRgbImage & input, FloatRgbImage & output, float a = 0.27f, float Lwhite = 3.0f);
	bool m_slow_tonemapping;
	inline bool ToneMapping(FloatRgbImage & input, FloatRgbImage & output, float a = 0.36f, float Lwhite = 3.0f){
			if( m_slow_tonemapping )
				return SlowToneMapping(input, output, a, Lwhite);
			else
				return FastToneMapping(input, output, a, Lwhite);
	}

	
	////////Gross Specular/////////////////
public: //Gross Specular
	bool m_do_gross_spec;
	bool GrossSpecular();
	GROSS_SPEC_INFO m_gross_spec_info;
	FloatImage m_gross_intensity;
	RgbImage m_gross_previmg;


	////////preseving Highlights///////////
public:
	bool m_do_hightlights;

	bool DetectHeightLightsPel(float Lmin, bool useHis);
	RgbImage m_highlights_img;
	float m_highlight_lmin;

	////////Inpaint////////////////////////
public:
	bool m_do_inpaint;
	bool Inpaint();
	FloatRgbImage m_inpainted_blur_img;  // no blur img
	FloatRgbImage m_inpainted_img;  // no blur img
	RgbImage m_inpaint_previmg;
	float m_g_value;


	////////Retexture//////////////////////
public:
	bool m_do_retexture;
	bool Retexture();
	bool Retex_Transparent();
	bool Retex_External();
	RETEX_INFO m_retex_info;
	FloatRgbImage m_retex_img;
	RgbImage m_retex_previmg;
	
	//float m_scalar_s;
	//float m_scalar_f;
	//float m_scalar_tex;
	//float m_tex_size;
	FloatRgbImage m_external_tex;

	////////Shape Recovery/////////////////
	// Results : m_gradient_img, m_depth_img, m_normal_img, m_brdf_img

public:
	ShapeEstimator	shape_estimator;		// Parameters
											//	float vars;
											//	float varl;
											//	float n;
											//	int recursion;

	bool			m_do_shape;	
	bool			m_use_gross;
	//Output
	FloatRgbImage	m_gradient_img;			// R = dx, G = dy
	FloatImage		m_depth_img;			// Range: 0~1
	FloatRgbImage	m_normal_img;			// R = x, G = y, B = z

	RgbImage		m_depth_previmg;		// preview image of the depth 0~1 => 0~255


	////////BRDF//////////////////////////
	bool			m_do_brdf;
	BRDF_Engine		m_brdf;
	FloatRgbImage	m_brdf_img;			// Final result before tone mapping
	RgbImage		m_brdf_previmg;
	BRDF_Engine brdf;					// Parameters
										//	float rho_d;
										//	float rho_s;
										//	float rms;
										//	vector3			*d;	brdf_samples = 200 samples 
										//	FloatRgbImage	l;	

};

extern ImgMatEdit g_ImgMatEdit;
