#include "Matrix.h"

void vector3::set(float xin, float yin, float zin)
{
	x = xin;
	y = yin;
	z = zin;
}

void vector3::middle(vector3 *i, vector3 *r)
{
	x = i->x + r->x;
	y = i->y + r->y;
	z = i->z + r->z;
	normalize();
}

void vector3::normalize()
{
	float norm = sqrt(x*x + y*y + z*z);
	x /= norm;
	y /= norm;
	z /= norm;
}

void vector3::rotate(matrix3 *r, vector3 *v)
{
	x = r->m00*v->x + r->m01*v->y + r->m02*v->z;
	y = r->m10*v->x + r->m11*v->y + r->m12*v->z;
	z = r->m20*v->x + r->m21*v->y + r->m22*v->z;
}

float dot(vector3 *v1, vector3 *v2)
{
	return v1->x*v2->x + v1->y*v2->y + v1->z*v2->z;
}

void cross_product(vector3 *a, vector3 *b, vector3 *c)
{
	c->x = a->y*b->z - a->z*b->y;
	c->y = a->z*b->x - a->x*b->z;
	c->z = a->x*b->y - a->y*b->x;
}

void matrix3::set(float alpha, vector3 *v)
{
	float a = alpha/2;
	float sina = sin(a);
	float q0 = cos(a);
	float q1 = v->x * sina;
	float q2 = v->y * sina;
	float q3 = v->z * sina;

	float q00 = q0*q0;	float q11 = q1*q1;	float q22 = q2*q2;	float q33 = q3*q3;	
	float q01 = q0*q1;	float q02 = q0*q2;	float q03 = q0*q3;
	float q12 = q1*q2;	float q13 = q1*q3;	float q23 = q2*q3;

	m00 = q11 - q22 - q33 + q00;	m01 = 2*(q12 - q03);			m02 = 2*(q13 + q02);
	m10 = 2*(q12 + q03);			m11 = -q11 + q22 - q33 + q00;	m12 = 2*(q23 - q01);
	m20 = 2*(q13 - q02);			m21 = 2*(q23 + q01);			m22 = -q11 - q22 + q33 + q00;

}

void matrix3::inverse(matrix3 *r)
{

}

vector3::vector3()
{
	x = 0;
	y = 0;
	z = 0;
}

vector3::~vector3(){}

matrix3::matrix3()
{
	m00 = m01 = m02 = 0;
	m10 = m11 = m12 = 0;
	m20 = m21 = m22 = 0;
}

matrix3::~matrix3(){}