#include "ShapeEstimator.h"
#include "ImgMatEdit.h"

void ShapeEstimator::SigmoidalCompression(FloatImage *L, RgbImage *matte)
{
	// Equation (7)(8)
	int x, y;
	int count;
	float temp;
	log_avg = 0;
	count = 0;
	for(y = 0; y < L->m_height; y++)
	{
		for(x = 0; x < L->m_width; x++)
		{
			if(matte->entry(x,y).R != 0)
			{
				log_avg += logf(L->entry(x, y) + (float)0.00000001);
				count++;
			}
		}
	}
	log_avg /= count;
	log_avg = exp(log_avg);
	log_avg = pow(log_avg, n);										// Result of equation (8)
	for(y = 0; y < L->m_height; y++)
	{
		for(x = 0; x < L->m_width; x++)
		{
			if(matte->entry(x,y).R != 0)
			{
				temp = (float)pow(L->entry(x,y), n);
				Ls->entry(x,y) = (float)(temp/(temp + log_avg));	// Equation (7)
			}			
		}
	}
}

void ShapeEstimator::DepthEstimation(FloatImage *depth, RgbImage *matte)
{
	// Equation (5)(6)
	int x, y, u, v;
	int w;
	float d;
	float v_curr;
	float value;
	float weight;
	float total_weight;

	// Re-calculate region of support here
	
	FloatImage filter = GaussianFilter2D(pow(vars * depth->m_width, (float)2.0));
	w = (filter.m_width-1)/2;
	for(y = 0; y < depth->m_height; y++)
	{
		for(x = 0; x < depth->m_width; x++)
		{
			v_curr = Ls->entry(x, y);
			if(matte->entry(x,y).R != 0)
			{
				total_weight = 0;	
				d = 0;
				
				for(v = -w; v <= w; v++)
				{
					for(u = -w; u <= w; u++)
					{
						value = Ls->entry(x-u, y-v);

						weight = (float)(filter.entry(u+w, v+w) * Gaussian(value-v_curr, varl));

						d += value*weight;
						total_weight += weight;
					}
				}
				depth->entry(x,y) = d/total_weight;	
			}
			else
			{
				depth->entry(x,y) = 0.0;
			}
		}
	}
}

void ShapeEstimator::InverseSigmoidal(FloatImage *depth, RgbImage *matte)
{
	// Equation (9)
	int x, y;
	float temp;
	float inverse_n = (float)1.0/n;
	if(d2 != NULL) delete d2;
		d2 = new FloatImage(depth->m_width, depth->m_height);
	for(y = 0; y < depth->m_height; y++)
	{
		for(x = 0; x < depth->m_width; x++)
		{
			if(matte->entry(x,y).R != 0)
			{
				temp = depth->entry(x,y);
				temp = pow(float(-1*log_avg*temp/(temp-1.0)), inverse_n);
				d2->entry(x,y) = (float)temp;
			}
			else
			{
				d2->entry(x,y) = 0.0;
			}
		}
	}
}

void ShapeEstimator::CalculateGradient(FloatRgbImage *gradient, RgbImage *matte)
{
	// Equation (10)
	int x, y;
	for(y = 0; y < d2->m_height; y++)
	{
		for(x = 0; x < d2->m_width; x++)
		{
			if(matte->entry(x,y).R != 0)
			{
				gradient->entry(x,y).R = d2->entry(x+1,y)-d2->entry(x,y);
				gradient->entry(x,y).G = d2->entry(x,y+1)-d2->entry(x,y);
				gradient->entry(x,y).R = reshape(gradient->entry(x,y).R);
				gradient->entry(x,y).G = reshape(gradient->entry(x,y).G);
			}
			else
			{
				gradient->entry(x,y).R = 0.0;
				gradient->entry(x,y).G = 0.0;
			}
		}
	}
}

void ShapeEstimator::CalculateGradient_Normal(FloatRgbImage *gradient, RgbImage *matte)
{
	// Equation (10)
	int x, y;
	float gxmax = 0.0;
	float gymax = 0.0;
	for(y = 0; y < d2->m_height; y++)
	{
		for(x = 0; x < d2->m_width; x++)
		{
			if(matte->entry(x,y).R != 0)
			{
				gradient->entry(x,y).R = d2->entry(x+1,y)-d2->entry(x,y);
				gradient->entry(x,y).G = d2->entry(x,y+1)-d2->entry(x,y);
				if(fabs(gradient->entry(x,y).R) > gxmax)
					gxmax = fabs(gradient->entry(x,y).R);
				if(fabs(gradient->entry(x,y).G) > gymax)
					gymax = fabs(gradient->entry(x,y).G);

			}
			else
			{
				gradient->entry(x,y).R = 0.0;
				gradient->entry(x,y).G = 0.0;
			}
		}
	}

	for(y = 0; y < d2->m_height; y++)
	{
		for(x = 0; x < d2->m_width; x++)
		{
			if(matte->entry(x,y).R != 0)
			{
				gradient->entry(x,y).R = reshape(gradient->entry(x,y).R/gxmax) *gxmax;
				gradient->entry(x,y).G = reshape(gradient->entry(x,y).G/gymax) *gymax;
			}
		}
	}
}

float ShapeEstimator::reshape(float in)
{
	float out;
	float t1;
	int i;
	if(in < 0)
	{
		out = -in;
		for(i = 0; i < recursion; i++)
		{
			t1 = out*out;
			out = 3*out - 6*t1 + 4*t1*out;
		}
		out = out * float(-1.0);
	}
	else
	{
		out = in;
		for(i = 0; i < recursion; i++)
		{
			t1 = out*out;
			out = 3*out - 6*t1 + 4*t1*out;
		}
	}

	return out;
}

void ShapeEstimator::CalculateNormal(FloatRgbImage *gradient, FloatRgbImage *normal, RgbImage *matte)
{
	// Equation (11)(12)
	int x, y;
	float length;
	for(y = 0; y < gradient->m_height; y++)
	{
		for(x = 0; x < gradient->m_width; x++)
		{
			if(matte->entry(x,y).R != 0)
			{
				normal->entry(x,y).R = -gradient->entry(x,y).G;
				normal->entry(x,y).G = -gradient->entry(x,y).R;
				normal->entry(x,y).B = 1.0;	
				//Normalize the length
				length =  normal->entry(x,y).R*normal->entry(x,y).R 
						+ normal->entry(x,y).G*normal->entry(x,y).G 
						+ normal->entry(x,y).B*normal->entry(x,y).B;

				length = (float)(1.0/sqrt(length));

				normal->entry(x,y).R *= length;
				normal->entry(x,y).G *= length;
				normal->entry(x,y).B *= length;
			}
			else
			{
				normal->entry(x,y).R = 0.0;
				normal->entry(x,y).G = 0.0;
				normal->entry(x,y).B = 1.0;
			}
		}
	}
}

void ShapeEstimator::Estimate(FloatImage *L, RgbImage *matte, 
							  FloatImage *depth, 
							  FloatRgbImage *gradient, 
							  FloatRgbImage *normal)
{
	g_ImgMatEdit.m_do_shape = false;

	if(Ls != NULL)			delete Ls;	

	depth->resize(L->m_width, L->m_height);
	normal->resize(L->m_width, L->m_height);
	gradient->resize(L->m_width, L->m_height);
	Ls		= new FloatImage(L->m_width, L->m_height);
	

	SigmoidalCompression(L, matte);
	//DepthEstimation(depth, matte);	//Slow verison
	DepthEstimation_Turbo(depth, matte);
	
	InverseSigmoidal(depth, matte);
	CalculateGradient_Normal(gradient, matte);
	CalculateNormal(gradient, normal, matte);

	//Debug(L, matte, depth, gradient, normal);
	g_ImgMatEdit.m_do_shape = true;
}

void ShapeEstimator::SetParameter(float  VarianceS, float  VarianceL, float  Exponent_n, int RecursionNum)
{
	vars		= VarianceS;
	varl		= VarianceL;
	n			= Exponent_n;
	recursion	= RecursionNum;
}


void ShapeEstimator::CreatePreview(FloatImage *depth, RgbImage *matte, RgbImage *preview)
{
	int x, y;
	preview->resize(matte->m_width, matte->m_height);
	for(y = 0; y < depth->m_height; y++)
	{
		for(x = 0; x < depth->m_width; x++)
		{
			if(matte->entry(x,y).R != 0)
			{
				preview->entry(x,y).R = preview->entry(x,y).G = preview->entry(x,y).B = (unsigned char)(depth->entry(x,y)*255);
			}
			else
			{
				preview->entry(x,y).R = preview->entry(x,y).G = preview->entry(x,y).B = 0;
			}
		}
	}
}

ShapeEstimator::ShapeEstimator(float  VarianceS, float  VarianceL, float  Exponent_n, int RecursionNum)
{
	SetParameter(VarianceS, VarianceL, Exponent_n, RecursionNum);
}

ShapeEstimator::ShapeEstimator()
{
	Ls = NULL;
	recursion	= 1;
	vars		= (float)0.004;
	varl		= (float)0.1;
	n			= (float)0.73;
}

ShapeEstimator::~ShapeEstimator()
{

}

void ShapeEstimator::DepthEstimation_Turbo(FloatImage *depth, RgbImage *matte)
{
	// Equation (5)(6)
	int x, y, u, v;
	int w;
	float d;
	float v_curr;
	float value;
	float weight;
	float total_weight;

	// Re-calculate region of support here
	
	FloatImage filter = GaussianFilter2D(pow(vars * depth->m_width, (float)2.0));
	w = (filter.m_width-1)/2;
	
	// Generate the intensity Gaussian filter table
	float *gauss_l = new float [256];
	for(x = 0; x < 255; x++)
	{
		gauss_l[x] = Gaussian((float)(x/255.0), varl);
	}

	for(y = 0; y < depth->m_height; y++)
	{
		for(x = 0; x < depth->m_width; x++)
		{
			v_curr = Ls->entry(x, y);
			if(matte->entry(x,y).R != 0)
			{
				total_weight = 0;	
				d = 0;
				
				for(v = -w; v <= w; v++)
				{
					for(u = -w; u <= w; u++)
					{
						value = Ls->entry(x-u, y-v);

						weight = (float)(filter.entry(u+w, v+w) * gauss_l[(int)(abs(value-v_curr) * 255)]);

						d += value*weight;
						total_weight += weight;
					}
				}
				depth->entry(x,y) = d/total_weight;	
			}
			else
			{
				depth->entry(x,y) = 0.0;
			}
		}
	}
	delete [] gauss_l;
}

void ShapeEstimator::Debug(FloatImage *L, RgbImage *matte, FloatImage *depth, FloatRgbImage *gradient, FloatRgbImage *normal)
{
	int x, y;
	FILE *debug;
	
	/*debug = fopen("lin.txt", "w");	
	{
	for(y = 0; y < L->m_height; y++)
	{
		for(x = 0; x < L->m_width; x++)
		{
			fprintf(debug, "%10f ", L->entry(x,y));
		}
		fprintf(debug, "\n");
	}
	fclose(debug);
	}

	debug = fopen("matte.txt", "w");
	{
	for(y = 0; y < L->m_height; y++)
	{
		for(x = 0; x < L->m_width; x++)
		{
			fprintf(debug, "%10d ", matte->entry(x,y).R);
		}
		fprintf(debug, "\n");
	}
	fclose(debug);
	}*/

	/*debug = fopen("ls.txt", "w");
	{
	for(y = 0; y < L->m_height; y++)
	{
		for(x = 0; x < L->m_width; x++)
		{
			fprintf(debug, "%10f ", Ls->entry(x,y));
		}
		fprintf(debug, "\n");
	}
	fclose(debug);
	}
	*/
	debug = fopen("depth.txt", "w");
	{
	for(y = 0; y < L->m_height; y++)
	{
		for(x = 0; x < L->m_width; x++)
		{
			fprintf(debug, "%10f ", depth->entry(x,y));
		}
		fprintf(debug, "\n");
	}
	fclose(debug);
	}

	debug = fopen("d2.txt", "w");
	for(y = 0; y < L->m_height; y++)
	{
		for(x = 0; x < L->m_width; x++)
		{
			fprintf(debug, "%10f ", d2->entry(x,y));
		}
		fprintf(debug, "\n");
	}
	fclose(debug);

	debug = fopen("gradient_x.txt", "w");
	for(y = 0; y < L->m_height; y++)
	{
		for(x = 0; x < L->m_width; x++)
		{
			fprintf(debug, "%10f ", gradient->entry(x,y).R);
		}
		fprintf(debug, "\n");
	}
	fclose(debug);

	debug = fopen("gradient_y.txt", "w");
	for(y = 0; y < L->m_height; y++)
	{
		for(x = 0; x < L->m_width; x++)
		{
			fprintf(debug, "%10f ", gradient->entry(x,y).G);
		}
		fprintf(debug, "\n");
	}
	fclose(debug);
}