/*
Shape Estimation

The operations described in Section 2 in [1]

Reference:
	[1] Khan, E. A. et al, Image-Based Materal Editing, in SIGGRAPH 2006

															Chia-Kai Liang
															June 13, 2006
*/
#pragma once
#include <cstdlib>
#include <cstdio>
#include <cmath>

using namespace std;

#include "stdafx.h"


class ShapeEstimator
{
public:
	ShapeEstimator();
	ShapeEstimator(float  VarianceS, float  VarianceL, float  Exponent_n, int RecursionNum);
	~ShapeEstimator();
	
	void SetParameter(float  VarianceS, float  VarianceL, float  Exponent_n, int RecursionNum);
	
	// Main function called by upper class
	void Estimate(FloatImage *L, RgbImage *matte, FloatImage *depth, FloatRgbImage *gradient, FloatRgbImage *normal);
	void Debug(FloatImage *L, RgbImage *matte, FloatImage *depth, FloatRgbImage *gradient, FloatRgbImage *normal);
	
	// Generate preview image 
	void CreatePreview(FloatImage *depth, RgbImage *matte, RgbImage *preview);

	void SigmoidalCompression(FloatImage *L, RgbImage *matte);								// Equation (7)(8)
	void DepthEstimation(FloatImage *depth, RgbImage *matte);								// Equation (5)(6)
	void DepthEstimation_Turbo(FloatImage *depth, RgbImage *matte);							// Equation (5)(6), speed up
	void InverseSigmoidal(FloatImage *depth, RgbImage *matte);								// Equation (9)
	void CalculateGradient(FloatRgbImage *gradient, RgbImage *matte);						// Equation (10)
	void CalculateGradient_Normal(FloatRgbImage *gradient, RgbImage *matte);				// Equation (10), reshape function is normalized
	void CalculateNormal(FloatRgbImage *gradient, FloatRgbImage *normal, RgbImage *matte);	// Equation (11)(12)
	
	float reshape(float in);

	FloatImage *Ls;
	FloatImage *d2;
	float vars;
	float varl;
	float n;
	float log_avg;
	int recursion;
	bool fast_estimate;
};