//*********************************************************************
//** Module       : path_conv.cpp
//** Directory    : ---
//** Class        : ---
//** Author/Date  : Boby Thomas/RBIN / 04/2006
//** Description  : Defines functions for converting relative path to
//**				absolute and vice versa.
//*********************************************************************


#include "path_conv.h"

//------------------------------------------------------------------------------
// Method		: Abs2Rel
// Description	: Convert absolute path to relative path.
// Parameter	: pcAbsPath - Input - Absolute path
// Parameter	: pcRelPath - Output - Relative path
// Parameter	: pcCurrDir - Input - Current Dir/Reference dir path
// Return		: Relative path
// Author		: Boby Thomas Pazheparampil April 2006
//------------------------------------------------------------------------------
char * Abs2Rel(char *pcAbsPath, char *pcRelPath, char* pcCurrDir)
{

	char acTmpCurrDir[MAX_PATH];
	char acTmpAbsPath[MAX_PATH];
	strcpy(acTmpCurrDir,pcCurrDir);
	strcpy(acTmpAbsPath,pcAbsPath);
	
	StackPtrChar tmpStackAbsPath;
	StackPtrChar tmpStackCurrPath;
	StackPtrChar tmpStackOutput;
	QueuePtrChar tmpMatchQueue;
	
	char *sTmp = strtok(acTmpAbsPath,path_separator);
	while(sTmp)
	{
		tmpStackAbsPath.push(sTmp);
		sTmp = strtok(0, path_separator);
	}

	sTmp = strtok(acTmpCurrDir,path_separator);
	while(sTmp)
	{
		tmpStackCurrPath.push(sTmp);
		sTmp = strtok(0, path_separator);
	}

	sTmp = pcRelPath;
	while(tmpStackCurrPath.size() > tmpStackAbsPath.size() )
	{
		*sTmp++ = '.';
		*sTmp++ = '.';
		*sTmp++ = path_sep_char;
		tmpStackCurrPath.pop();
	}

	while(tmpStackAbsPath.size() > tmpStackCurrPath.size() )
	{
		char *pcTmp = tmpStackAbsPath.top();
		tmpStackOutput.push(pcTmp);
		tmpStackAbsPath.pop();
	}

	while(tmpStackAbsPath.size() > 0)
	{
		if(strcmp(tmpStackAbsPath.top(),tmpStackCurrPath.top())== 0  )
			tmpMatchQueue.push(tmpStackAbsPath.top());
		else
		{
			while(tmpMatchQueue.size() > 0)
				tmpStackOutput.push(tmpMatchQueue.front());
			tmpStackOutput.push(tmpStackAbsPath.top());
			*sTmp++ = '.';
			*sTmp++ = '.';
			*sTmp++ = path_sep_char;
		}
		tmpStackAbsPath.pop();
		tmpStackCurrPath.pop();	
	}
	while(tmpStackOutput.size() > 0)
	{
		char *pcTmp= tmpStackOutput.top();
		while(*pcTmp != '\0')	
			*sTmp++ = *pcTmp++;
		tmpStackOutput.pop();
		*sTmp++ = path_sep_char;
	}
	*(--sTmp) = '\0';

	return pcRelPath;
}

//------------------------------------------------------------------------------
// Method		: Rel2Abs
// Description	: Convert absolute path to relative path.
// Parameter	: pcRelPath - Input - Relative path
// Parameter	: pcAbsPath - Output - Absolute path
// Parameter	: pcCurrDir - Input - Current Dir/Reference dir path
// Return		: Absolute path
// Author		: Boby Thomas Pazheparampil April 2006
//------------------------------------------------------------------------------

char * Rel2Abs(char *pcRelPath, char *pcAbsPath, char* pcCurrDir)
{
	char acTmpCurrDir[MAX_PATH];
	char acTmpRelPath[MAX_PATH];
	strcpy(acTmpCurrDir,pcCurrDir);
	strcpy(acTmpRelPath,pcRelPath);
	
	QueuePtrChar tmpQueueRelPath;
	StackPtrChar tmpStackCurrPath;
	StackPtrChar tmpStackOutPath;
	
	char *sTmp = strtok(acTmpRelPath,path_separator);
	while(sTmp)
	{
		tmpQueueRelPath.push(sTmp);
		sTmp = strtok(0, path_separator);
	}

	sTmp = strtok(acTmpCurrDir,path_separator);
	while(sTmp)
	{
		tmpStackCurrPath.push(sTmp);
		sTmp = strtok(0, path_separator);
	}


	while(tmpQueueRelPath.size() > 0)
	{
		char *pcTmp= tmpQueueRelPath.front();
		if(strcmp(pcTmp, "..") == 0)
			tmpStackCurrPath.pop();
		else
			tmpStackCurrPath.push(pcTmp);
		tmpQueueRelPath.pop();
	}
	while(tmpStackCurrPath.size() > 0)
	{
		tmpStackOutPath.push(tmpStackCurrPath.top());
		tmpStackCurrPath.pop();
	}


	sTmp = pcAbsPath;
#if defined(__GNUC__)
	*sTmp++ = path_sep_char;
#endif
	while(tmpStackOutPath.size() > 0)
	{
		char *pcTmp= tmpStackOutPath.top();
		while(*pcTmp != '\0')	
			*sTmp++ = *pcTmp++;
		tmpStackOutPath.pop();
		*sTmp++ = path_sep_char;
	}
	*(--sTmp) = '\0';

	return pcAbsPath; 	
}


//------------------------------------------------------------------------------
// Method		: GetCurrentDir
// Description	: Get Current Directory.
// Parameter	: pcTmp - Output - Current directory
// Return		: ---
// Author		: Boby Thomas Pazheparampil April 2006
//------------------------------------------------------------------------------
void GetCurrentDir(char* pcTmp)
{

#if defined(_MSC_VER)
   GetCurrentDirectory(MAX_PATH,pcTmp);
#elif defined(__GNUC__)
   getcwd(pcTmp,MAX_PATH);
#else
	#error define your compiler
#endif

   return;
}

